/* 
 * empty.c --
 *
 *	This file contains the C-APIs for reversing a list.
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: empty.c,v 1.1 2000/01/14 23:44:16 aku Exp $
 */

#include "listxInt.h"


/*
 *----------------------------------------------------------------------
 *
 * Listx_Empty --
 *
 *	Checks the specified list object for emptiness.
 *
 * Results:
 *	A standard Tcl result. Writes the boolean result into *emptyPtr
 *
 * Side effects:
 *	See above.
 *
 *----------------------------------------------------------------------
 */

int
Listx_Empty (interp, listObj, emptyPtr)
     Tcl_Interp* interp;
     Tcl_Obj*    listObj;
     int*        emptyPtr;
{
    int listc, result;
    Tcl_Obj **listv;

    /*
     * Make sure the list argument is a list object and get its length and
     * a pointer to its array of element pointers.
     */

    result = Tcl_ListObjGetElements(interp, listObj, &listc, &listv);

    if (result != TCL_OK) {
	return result;
    }

    *emptyPtr = (listc == 0);
    return TCL_OK;
}

