/* -*- c -*-
 *
 * init.c --
 *
 *	Implements the C level procedures handling the initialization of
 *	this package.
 *
 *
 * Copyright (c) 1999 Andreas Kupries (a.kupries@westend.com)
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL I LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS
 * SOFTWARE AND ITS DOCUMENTATION, EVEN IF I HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * I SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
 * I HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 *
 * CVS: $Id: init.c,v 1.2 2000/01/14 23:44:16 aku Exp $
 */

#include "listxInt.h"
#include "listxFind.h"

#define NEW_NS(n) \
Tcl_VarEval (interp, "namespace eval ", (n), " {}", (char*) NULL)


/*
 *------------------------------------------------------*
 *
 *	Listx_Init --
 *
 *	------------------------------------------------*
 *	Standard procedure required by 'load'. 
 *	Initializes this extension.
 *	------------------------------------------------*
 *
 *	Sideeffects:
 *		As of 'Tcl_CreateCommand'.
 *
 *	Result:
 *		A standard Tcl error code.
 *
 *------------------------------------------------------*
 */

EXPORT (int,Listx_Init) (interp)
     Tcl_Interp* interp;
{
#if 0
    Tcl_Namespace* ns;
#endif
    Tcl_Obj*       unknown;
    Tcl_CmdInfo    info;

    if (Tcl_InitStubs(interp, "8.2", 0) == NULL) {
        return TCL_ERROR;
    }

    Tcl_RegisterObjType (&selectorObjType);

#if 0
    /*
     * Create the ::tcl namespace if it should prove necessary.
     */

    ns = Tcl_FindNamespace (interp, "::listx",
			    (Tcl_Namespace *) NULL, TCL_GLOBAL_ONLY);

    if (ns == (Tcl_Namespace *) NULL) {
        ns = Tcl_CreateNamespace (interp, "::tcl", (ClientData) NULL,
				  (Tcl_NamespaceDeleteProc *) NULL);

	if (ns == (Tcl_Namespace *) NULL) {
	    return TCL_ERROR;
	}
    }
#endif

    if (TCL_OK != NEW_NS ("::listx")) {
        return TCL_ERROR;
    }

    if (TCL_OK != Tcl_Eval (interp, FIND_LIBRARY)) {
        return TCL_ERROR;
    }

    Tcl_CreateObjCommand (interp, "::listx", &ListxListxObjCmd,
			  (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL);

    ListxListType = Tcl_GetObjType ("list");
    ListxEmptyObj = Tcl_NewObj (); /* Memory leak, never freed */
    ListxIntType  = Tcl_GetObjType ("int");

    unknown = Tcl_NewStringObj("::listx::unknown", -1);
    Tcl_IncrRefCount (unknown);

    if (Tcl_GetCommandInfo(interp, Tcl_GetString (unknown), &info)) {
       ListxUnknownCmdObj = unknown;
    } else {
       Tcl_DecrRefCount (unknown);
    }

    /* register list extension as available package */
    Tcl_PkgProvide (interp, "Listx", LISTX_VERSION);
    return TCL_OK;
}

/*
 *------------------------------------------------------*
 *
 *	Listx_SafeInit --
 *
 *	------------------------------------------------*
 *	Standard procedure required by 'load'. 
 *	Initializes this extension for a safe interpreter.
 *	------------------------------------------------*
 *
 *	Sideeffects:
 *		As of 'Listx_Init'
 *
 *	Result:
 *		A standard Tcl error code.
 *
 *------------------------------------------------------*
 */

EXPORT (int,Listx_SafeInit) (interp)
     Tcl_Interp* interp;
{
  return Listx_Init (interp);
}

