#ifndef LISTX_H
#define LISTX_H
/* -*- c -*-
 *
 * transform.h - externally visible facilities of data transformers
 *
 * Distributed at @mDate@.
 *
 * Copyright (c) 1995 Andreas Kupries (a.kupries@westend.com)
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL I BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
 * INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS
 * SOFTWARE AND ITS DOCUMENTATION, EVEN IF I HAVE BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * I SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND
 * I HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 * ENHANCEMENTS, OR MODIFICATIONS.
 *
 * CVS: $Id: listx.h,v 1.2 2000/01/14 23:44:16 aku Exp $
 */

#ifdef __cplusplus
extern "C" {
#endif

#include <tcl.h>

#if defined(__WIN32__)
#   define WIN32_LEAN_AND_MEAN
#   include <windows.h>
#   undef WIN32_LEAN_AND_MEAN

/*
 * VC++ has an alternate entry point called DllMain, so we need to rename
 * our entry point.
 */


#ifdef TCL_STORAGE_CLASS
# undef TCL_STORAGE_CLASS
#endif
#ifdef BUILD_Listx
# define TCL_STORAGE_CLASS DLLEXPORT
#else
# define TCL_STORAGE_CLASS DLLIMPORT
#endif

#   if defined(_MSC_VER)
#	define LISTX_EXPORT(a,b) TCL_STORAGE_CLASS a b
#   else
#	if defined(__BORLANDC__)
#	    define LISTX_EXPORT(a,b) a _export b
#	else
#	    define LISTX_EXPORT(a,b) a b
#	endif
#   endif
#else
#   define LISTX_EXPORT(a,b) a b
#endif

/*
 * Exported tcl level procedures.
 *
 * ATTENTION:
 * due to the fact that cpp - processing with gcc 2.5.8 removes any comments
 * in macro-arguments (even if called with option '-C') i have to use the
 * predefined macro __C2MAN__ to distinguish real compilation and manpage
 * generation, removing _ANSI_ARGS_ in the latter case.
 */

/*
 * Initialize extension in standard interpreter.
 * Extends the interpreter with extension-specific
 * structures and installs the globally visible
 * command of Tcl-LISTX.  Will catch attempts for
 * multiple initialization of an interpreter.
 */

#ifdef __C2MAN__
int
Listx_Init (Tcl_Interp* interp /* interpreter to initialize */);
#else
LISTX_EXPORT (int,Listx_Init) _ANSI_ARGS_ ((Tcl_Interp* interp));
#endif

/*
 * Initialize extension in SAFE interpreter.
 * Same as --> Listx_Init.  The only security
 * relevant operations are reading from and
 * writing to a file.  As Tcl-Handles are
 * given to these commands it is assumed that
 * they were checked and cleared beforehand.
 */

#ifdef __C2MAN__
int
Listx_SafeInit (Tcl_Interp* interp /* interpreter to initialize */);
#else
LISTX_EXPORT (int,Listx_SafeInit) _ANSI_ARGS_ ((Tcl_Interp* interp));
#endif


/*
 * An enumeration of the possible modes for Listx_Search.
 */

typedef enum lSearchModes {
  LSEARCH_EXACT,  /* -exact  */
  LSEARCH_GLOB,   /* -glob   */
  LSEARCH_REGEXP  /* -regexp */
} lSearchModes;

/*
 * Interface of the functions used for mapping and or folding objects.
 * Used by Listx_Map / Listx_Fold.
 */

typedef int Listx_MapProc _ANSI_ARGS_ ((ClientData clientData,
	 Tcl_Interp* interp, Tcl_Obj* inObj, Tcl_Obj** resultObj));

typedef int Listx_FoldProc _ANSI_ARGS_ ((ClientData clientData,
	 Tcl_Interp* interp, Tcl_Obj* accumulatorObj, Tcl_Obj* inObj,
	 Tcl_Obj** resultObj));

/*
 * A structure to contain selector information for 'index' and 'remove'.
 *
 * 1) Single index (number)
 * 2) Array of indices (1).
 * 3) Ranges (start index, end index and stride)
 *
 * A flag is used to distinguish between normal indices and such ranging
 * from the 'end'.
 */

typedef enum lSelectorType {
  LIST_SEL_SINGLE,
  LIST_SEL_MULTIPLE,
  LIST_SEL_RANGE
} lSelectorType;

typedef struct {
  long int location;   /* location of object, reelative to either start or
			* end. See the flag below. */
  int      offsetFrom; /* Flag: 0 = location from start, LOFF_START,
			*       1 = location from end,   LOFF_END */
} lIndex;

#define LOFF_START (0)
#define LOFF_END   (1)

typedef struct {
  lSelectorType type; /* What type of selector is stored in the structure */

  union {
    lIndex index;    /* type == LIST_SEL_SINGLE:
		      *    A single index */
    struct {         /* type == LIST_SEL_MULTIPLE: */
      lIndex start;  /*    The start of the range */
      lIndex end;    /*    The end of the range */
      int    stride; /*    The delta between elements in the range */
    } range;

    struct {         /* type == LIST_SEL_RANGE: */
      int      num;  /*    The number of items in the array */
      lIndex* index; /*    A dynamically alloc. array holding the indices */
    } multiple;
  } info;
} listSelector;


/*
 * And now the public interface of this extension.
 */

#ifndef __C2MAN__
#ifndef LISTX_USE_STUBS
#include "listxDecls.h"
#endif
#endif

/*
 * Convenience declaration of Listx_InitStubs.
 * This function is not *implemented* by the listx library, so the storage
 * class is neither DLLEXPORT nor DLLIMPORT
 */

#undef TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS

EXTERN char *Listx_InitStubs _ANSI_ARGS_((Tcl_Interp *interp,
					  CONST char *version, int exact));

#ifndef USE_LISTX_STUBS

#define Listx_InitStubs(interp, version, exact) \
    Tcl_PkgRequire(interp, "Listx", version, exact)

#endif

#undef  TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS DLLIMPORT

#ifdef __cplusplus
}
#endif /* C++ */
#endif /* LISTX_H */
