/* 
 * listxStubLib.c --
 *
 *	Stub object that will be statically linked into extensions that wish
 *	to access Listx.
 */


/*
 * We need to ensure that we use the stub macros so that this file contains
 * no references to any of the stub functions.  This will make it possible
 * to build an extension that references Listx_InitStubs but doesn't end up
 * including the rest of the stub functions.
 */

#ifndef USE_TCL_STUBS
#define USE_TCL_STUBS
#endif
#undef  USE_TCL_STUB_PROCS

#ifndef USE_LISTX_STUBS
#define USE_LISTX_STUBS
#endif
#undef  USE_LISTX_STUB_PROCS

#include "listx.h"
#include "listxIntDecls.h"

/*
 * Ensure that Listx_InitStubs is built as an exported symbol.  The other stub
 * functions should be built as non-exported symbols.
 */

#undef  TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS DLLEXPORT

ListxStubs*    listxStubsPtr;
ListxIntStubs* listxIntStubsPtr;


/*
 *----------------------------------------------------------------------
 *
 * Listx_InitStubs --
 *
 *	Checks that the correct version of Listx is loaded and that it
 *	supports stubs. It then initialises the stub table pointers.
 *
 * Results:
 *	The actual version of Listx that satisfies the request, or
 *	NULL to indicate that an error occurred.
 *
 * Side effects:
 *	Sets the stub table pointers.
 *
 *----------------------------------------------------------------------
 */

#ifdef Listx_InitStubs
#undef Listx_InitStubs
#endif

char *
Listx_InitStubs(interp, version, exact)
    Tcl_Interp *interp;
    CONST char *version;
    int exact;
{
    char *actualVersion;

    actualVersion = Tcl_PkgRequireEx(interp, "Listx", (char *) version, exact,
		(ClientData *) &listxStubsPtr);

    if (!actualVersion) {
	return NULL;
    }

    if (!listxStubsPtr) {
	Tcl_SetResult(interp,
		"This implementation of Listx does not support stubs",
		TCL_STATIC);
	return NULL;
    }
    
    listxIntStubsPtr = listxStubsPtr->hooks->listxIntStubs;
    
    return actualVersion;
}
