/* 
 * reverse.c --
 *
 *	This file contains the C-APIs for reversing a list.
 *
 * Copyright (c) 1987-1993 The Regents of the University of California.
 * Copyright (c) 1994-1997 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: reverse.c,v 1.2 2000/01/14 23:44:16 aku Exp $
 */

#include "listxInt.h"

/*
 *----------------------------------------------------------------------
 *
 * Listx_Reverse --
 *
 *	Reverses the order of the elements in the specified list object.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See above, invalidates the string representation.
 *
 *----------------------------------------------------------------------
 */

int
Listx_Reverse (interp, listObj, resultObj)
     Tcl_Interp* interp;
     Tcl_Obj*    listObj;
     Tcl_Obj**   resultObj;
{
    int listc, result, i, j;
    Tcl_Obj **listv, *tmp, *res;

    /*
     * Make sure the list argument is a list object and get its length and
     * a pointer to its array of element pointers.
     */

    result = Tcl_ListObjGetElements(interp, listObj, &listc, &listv);

    if (result != TCL_OK) {
	return result;
    }

    /*
     * All of the above was just preparation. Now we do the reversal
     * and give the result back to our caller.
     *
     * To avoid the allocation of a temporary array to hold the
     * reversed array we do the reversal twice, and create the new object
     * while in the reversed state.
     */

    for (i=0, j=listc-1; i < j; i++, j--) {
        tmp       = listv [i];
	listv [i] = listv [j];
	listv [j] = tmp;
    }

    res = Tcl_NewListObj (listc, listv);

    for (i=0, j=listc-1; i < j; i++, j--) {
        tmp       = listv [i];
	listv [i] = listv [j];
	listv [j] = tmp;
    }

    if (res == (Tcl_Obj*) NULL) {
      return TCL_ERROR;
    }

    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * Listx_ReverseInPlace --
 *
 *	Reverses the order of the elements in the specified list object.
 *
 * Results:
 *	A standard Tcl result.
 *
 * Side effects:
 *	See above, invalidates the string representation.
 *
 *----------------------------------------------------------------------
 */

int
Listx_ReverseInPlace (interp, listObj)
     Tcl_Interp* interp;
     Tcl_Obj*    listObj;
{
    int listc, result, i, j;
    Tcl_Obj **listv, *tmp;

    /*
     * Make sure the list argument is a list object and get its length and
     * a pointer to its array of element pointers.
     */

    result = Tcl_ListObjGetElements(interp, listObj, &listc, &listv);

    if (result != TCL_OK) {
	return result;
    }

    /*
     * All of the above was just preparation. Now we do the reversal
     * and give the result back to our caller. Don't forget to
     * invalidate the string representation.
     */

    for (i=0, j=listc-1; i < j; i++, j--) {
        tmp       = listv [i];
	listv [i] = listv [j];
	listv [j] = tmp;
    }

    Tcl_InvalidateStringRep (listObj);
    return TCL_OK;
}

